# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: worker.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cworker.proto\x1a\x1cgoogle/protobuf/struct.proto\">\n\x13LeaderUpdateRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x11\n\thost_port\x18\x02 \x01(\x03\"u\n\x1aWorkerServerUpdateResponse\x12\x16\n\x0e\x63lient_address\x18\x01 \x01(\t\x12\x13\n\x0b\x63lient_port\x18\x02 \x01(\x03\x12*\n\tcompleted\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x88\x01\n\rNewJobRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\x12+\n\njob_config\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x13\n\x0bjob_timeout\x18\x05 \x01(\x03\"G\n\x0eNewJobResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\"J\n\x11PollWorkerRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\"N\n\x12PollWorkerResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x11\n\tcompleted\x18\x03 \x01(\x08\"K\n\x12JobCompleteRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06job_id\x18\x03 \x01(\t\"j\n\x13JobCompleteResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12,\n\x0bjob_results\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\"O\n\x16WorkerHeartbeatRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"P\n\x17WorkerHeartbeatResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"D\n\x1bLeaderRegisterWorkerRequest\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\"]\n\x1cLeaderRegisterWorkerResponse\x12\x14\n\x0chost_address\x18\x01 \x01(\t\x12\x0f\n\x07host_id\x18\x02 \x01(\t\x12\x16\n\x0eserver_address\x18\x03 \x01(\t2\xaf\x03\n\x0cWorkerServer\x12K\n\x14GetRequestsCompleted\x12\x14.LeaderUpdateRequest\x1a\x1b.WorkerServerUpdateResponse\"\x00\x12\x37\n\x12\x43reateNewWorkerJob\x12\x0e.NewJobRequest\x1a\x0f.NewJobResponse\"\x00\x12\x39\n\x0cGetJobStatus\x12\x12.PollWorkerRequest\x1a\x13.PollWorkerResponse\"\x00\x12<\n\rGetJobResults\x12\x13.JobCompleteRequest\x1a\x14.JobCompleteResponse\"\x00\x12K\n\x14\x43heckWorkerHeartbeat\x12\x17.WorkerHeartbeatRequest\x1a\x18.WorkerHeartbeatResponse\"\x00\x12S\n\x14LeaderRegisterWorker\x12\x1c.LeaderRegisterWorkerRequest\x1a\x1d.LeaderRegisterWorkerResponseb\x06proto3')



_LEADERUPDATEREQUEST = DESCRIPTOR.message_types_by_name['LeaderUpdateRequest']
_WORKERSERVERUPDATERESPONSE = DESCRIPTOR.message_types_by_name['WorkerServerUpdateResponse']
_NEWJOBREQUEST = DESCRIPTOR.message_types_by_name['NewJobRequest']
_NEWJOBRESPONSE = DESCRIPTOR.message_types_by_name['NewJobResponse']
_POLLWORKERREQUEST = DESCRIPTOR.message_types_by_name['PollWorkerRequest']
_POLLWORKERRESPONSE = DESCRIPTOR.message_types_by_name['PollWorkerResponse']
_JOBCOMPLETEREQUEST = DESCRIPTOR.message_types_by_name['JobCompleteRequest']
_JOBCOMPLETERESPONSE = DESCRIPTOR.message_types_by_name['JobCompleteResponse']
_WORKERHEARTBEATREQUEST = DESCRIPTOR.message_types_by_name['WorkerHeartbeatRequest']
_WORKERHEARTBEATRESPONSE = DESCRIPTOR.message_types_by_name['WorkerHeartbeatResponse']
_LEADERREGISTERWORKERREQUEST = DESCRIPTOR.message_types_by_name['LeaderRegisterWorkerRequest']
_LEADERREGISTERWORKERRESPONSE = DESCRIPTOR.message_types_by_name['LeaderRegisterWorkerResponse']
LeaderUpdateRequest = _reflection.GeneratedProtocolMessageType('LeaderUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _LEADERUPDATEREQUEST,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:LeaderUpdateRequest)
  })
_sym_db.RegisterMessage(LeaderUpdateRequest)

WorkerServerUpdateResponse = _reflection.GeneratedProtocolMessageType('WorkerServerUpdateResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKERSERVERUPDATERESPONSE,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:WorkerServerUpdateResponse)
  })
_sym_db.RegisterMessage(WorkerServerUpdateResponse)

NewJobRequest = _reflection.GeneratedProtocolMessageType('NewJobRequest', (_message.Message,), {
  'DESCRIPTOR' : _NEWJOBREQUEST,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:NewJobRequest)
  })
_sym_db.RegisterMessage(NewJobRequest)

NewJobResponse = _reflection.GeneratedProtocolMessageType('NewJobResponse', (_message.Message,), {
  'DESCRIPTOR' : _NEWJOBRESPONSE,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:NewJobResponse)
  })
_sym_db.RegisterMessage(NewJobResponse)

PollWorkerRequest = _reflection.GeneratedProtocolMessageType('PollWorkerRequest', (_message.Message,), {
  'DESCRIPTOR' : _POLLWORKERREQUEST,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:PollWorkerRequest)
  })
_sym_db.RegisterMessage(PollWorkerRequest)

PollWorkerResponse = _reflection.GeneratedProtocolMessageType('PollWorkerResponse', (_message.Message,), {
  'DESCRIPTOR' : _POLLWORKERRESPONSE,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:PollWorkerResponse)
  })
_sym_db.RegisterMessage(PollWorkerResponse)

JobCompleteRequest = _reflection.GeneratedProtocolMessageType('JobCompleteRequest', (_message.Message,), {
  'DESCRIPTOR' : _JOBCOMPLETEREQUEST,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:JobCompleteRequest)
  })
_sym_db.RegisterMessage(JobCompleteRequest)

JobCompleteResponse = _reflection.GeneratedProtocolMessageType('JobCompleteResponse', (_message.Message,), {
  'DESCRIPTOR' : _JOBCOMPLETERESPONSE,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:JobCompleteResponse)
  })
_sym_db.RegisterMessage(JobCompleteResponse)

WorkerHeartbeatRequest = _reflection.GeneratedProtocolMessageType('WorkerHeartbeatRequest', (_message.Message,), {
  'DESCRIPTOR' : _WORKERHEARTBEATREQUEST,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:WorkerHeartbeatRequest)
  })
_sym_db.RegisterMessage(WorkerHeartbeatRequest)

WorkerHeartbeatResponse = _reflection.GeneratedProtocolMessageType('WorkerHeartbeatResponse', (_message.Message,), {
  'DESCRIPTOR' : _WORKERHEARTBEATRESPONSE,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:WorkerHeartbeatResponse)
  })
_sym_db.RegisterMessage(WorkerHeartbeatResponse)

LeaderRegisterWorkerRequest = _reflection.GeneratedProtocolMessageType('LeaderRegisterWorkerRequest', (_message.Message,), {
  'DESCRIPTOR' : _LEADERREGISTERWORKERREQUEST,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:LeaderRegisterWorkerRequest)
  })
_sym_db.RegisterMessage(LeaderRegisterWorkerRequest)

LeaderRegisterWorkerResponse = _reflection.GeneratedProtocolMessageType('LeaderRegisterWorkerResponse', (_message.Message,), {
  'DESCRIPTOR' : _LEADERREGISTERWORKERRESPONSE,
  '__module__' : 'worker_pb2'
  # @@protoc_insertion_point(class_scope:LeaderRegisterWorkerResponse)
  })
_sym_db.RegisterMessage(LeaderRegisterWorkerResponse)

_WORKERSERVER = DESCRIPTOR.services_by_name['WorkerServer']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LEADERUPDATEREQUEST._serialized_start=46
  _LEADERUPDATEREQUEST._serialized_end=108
  _WORKERSERVERUPDATERESPONSE._serialized_start=110
  _WORKERSERVERUPDATERESPONSE._serialized_end=227
  _NEWJOBREQUEST._serialized_start=230
  _NEWJOBREQUEST._serialized_end=366
  _NEWJOBRESPONSE._serialized_start=368
  _NEWJOBRESPONSE._serialized_end=439
  _POLLWORKERREQUEST._serialized_start=441
  _POLLWORKERREQUEST._serialized_end=515
  _POLLWORKERRESPONSE._serialized_start=517
  _POLLWORKERRESPONSE._serialized_end=595
  _JOBCOMPLETEREQUEST._serialized_start=597
  _JOBCOMPLETEREQUEST._serialized_end=672
  _JOBCOMPLETERESPONSE._serialized_start=674
  _JOBCOMPLETERESPONSE._serialized_end=780
  _WORKERHEARTBEATREQUEST._serialized_start=782
  _WORKERHEARTBEATREQUEST._serialized_end=861
  _WORKERHEARTBEATRESPONSE._serialized_start=863
  _WORKERHEARTBEATRESPONSE._serialized_end=943
  _LEADERREGISTERWORKERREQUEST._serialized_start=945
  _LEADERREGISTERWORKERREQUEST._serialized_end=1013
  _LEADERREGISTERWORKERRESPONSE._serialized_start=1015
  _LEADERREGISTERWORKERRESPONSE._serialized_end=1108
  _WORKERSERVER._serialized_start=1111
  _WORKERSERVER._serialized_end=1542
# @@protoc_insertion_point(module_scope)
